@@ -59,6 +59,7 @@ typedef struct {
         char *type;
         char *log;
         char *host;
+        char *file;
         char *port;
         char *name;
         char *user;
@@ -75,20 +76,17 @@ PRELUDE_PLUGIN_OPTION_DECLARE_STRING_CB(db, db_plugin_t, port)
 PRELUDE_PLUGIN_OPTION_DECLARE_STRING_CB(db, db_plugin_t, name)
 PRELUDE_PLUGIN_OPTION_DECLARE_STRING_CB(db, db_plugin_t, user)
 PRELUDE_PLUGIN_OPTION_DECLARE_STRING_CB(db, db_plugin_t, pass)
-
+PRELUDE_PLUGIN_OPTION_DECLARE_STRING_CB(db, db_plugin_t, file)
 
 
 static int db_run(prelude_plugin_instance_t *pi, idmef_message_t *message)
 {
-        db_plugin_t *plugin = prelude_plugin_instance_get_plugin_data(pi);
-        char errbuf[PRELUDEDB_ERRBUF_SIZE];
         int ret;
+        db_plugin_t *plugin = prelude_plugin_instance_get_plugin_data(pi);
 
         ret = preludedb_insert_message(plugin->db, message);
         if ( ret < 0 )
-                prelude_log(PRELUDE_LOG_WARN,
-                            "could not insert message into database: %s.\n",
-                            preludedb_get_error(plugin->db, ret, errbuf, sizeof(errbuf)));
+                prelude_log(PRELUDE_LOG_WARN, "could not insert message into database: %s.\n", preludedb_strerror(ret));
 
         return ret;
 }
@@ -105,6 +103,9 @@ static void db_destroy(prelude_plugin_instance_t *pi, prelude_string_t *out)
         if ( plugin->host )
                 free(plugin->host);
 
+        if ( plugin->file )
+                free(plugin->file);
+        
         if ( plugin->name )
                 free(plugin->name);
 
@@ -119,8 +120,10 @@ static void db_destroy(prelude_plugin_instance_t *pi, prelude_string_t *out)
 
         if ( plugin->log )
                 free(plugin->log);
-        
-        preludedb_destroy(plugin->db);
+
+        if ( plugin->db )
+                preludedb_destroy(plugin->db);
+
         free(plugin);
 }
 
@@ -132,9 +135,8 @@ static int db_init(prelude_plugin_instance_t *pi, prelude_string_t *out)
         preludedb_t *db;
         preludedb_sql_t *sql;
         preludedb_sql_settings_t *settings;
-        char errbuf[PRELUDEDB_ERRBUF_SIZE];
         db_plugin_t *plugin = prelude_plugin_instance_get_plugin_data(pi);
-                
+        
         ret = preludedb_sql_settings_new(&settings);        
         if ( ret < 0 )
                 return ret;
@@ -142,6 +144,9 @@ static int db_init(prelude_plugin_instance_t *pi, prelude_string_t *out)
         if ( plugin->host )
                 preludedb_sql_settings_set_host(settings, plugin->host);
 
+        if ( plugin->file )
+                preludedb_sql_settings_set_file(settings, plugin->file);
+        
         if ( plugin->port )
                 preludedb_sql_settings_set_port(settings, plugin->port);
 
@@ -161,8 +166,10 @@ static int db_init(prelude_plugin_instance_t *pi, prelude_string_t *out)
                 return ret;
         }
 
-	if ( plugin->log ) {
-		ret = preludedb_sql_enable_query_logging(sql, plugin->log);
+        if ( ! plugin->log )
+                preludedb_sql_disable_query_logging(sql);
+        else {
+                ret = preludedb_sql_enable_query_logging(sql, (strcmp(plugin->log, "-") == 0) ? NULL : plugin->log);
                 if ( ret < 0 ) {
 			preludedb_sql_destroy(sql);
                         prelude_string_sprintf(out, "could not enable queries logging with log file '%s': %s",
@@ -171,10 +178,10 @@ static int db_init(prelude_plugin_instance_t *pi, prelude_string_t *out)
 		}
 	}
 
-        ret = preludedb_new(&db, sql, NULL, errbuf, sizeof(errbuf));
+        ret = preludedb_new(&db, sql, NULL, NULL, 0);
         if ( ret < 0 ) {
 		preludedb_sql_destroy(sql);
-                prelude_string_sprintf(out, "could not initialize libpreludedb: %s", errbuf);
+                prelude_string_sprintf(out, "could not initialize libpreludedb: %s", preludedb_strerror(ret));
 		return ret;
 	}
 
@@ -191,7 +198,7 @@ static int db_init(prelude_plugin_instance_t *pi, prelude_string_t *out)
 static int db_activate(prelude_option_t *opt, const char *optarg, prelude_string_t *err, void *context) 
 {
         db_plugin_t *new;
-
+        
         new = calloc(1, sizeof(*new));
         if ( ! new )
                 return prelude_error_from_errno(errno);
@@ -237,7 +244,7 @@ int db_LTX_manager_plugin_init(prelude_plugin_entry_t *pe, void *rootopt)
         
 	ret = prelude_option_add(opt, NULL, hook, 'l', "log",
                                  "Log all queries in a file, should be only used for debugging purpose",
-                                 PRELUDE_OPTION_ARGUMENT_REQUIRED, db_set_log, db_get_log);
+                                 PRELUDE_OPTION_ARGUMENT_OPTIONAL, db_set_log, db_get_log);
         if ( ret < 0 )
                 return ret;
         
@@ -246,6 +253,12 @@ int db_LTX_manager_plugin_init(prelude_plugin_entry_t *pe, void *rootopt)
                                  PRELUDE_OPTION_ARGUMENT_REQUIRED,  db_set_host, db_get_host);
         if ( ret < 0 )
                 return ret;
+
+        ret = prelude_option_add(opt, NULL, hook, 'f', PRELUDEDB_SQL_SETTING_FILE,
+                                 "The file where the database is stored (in case of file based database)",
+                                 PRELUDE_OPTION_ARGUMENT_REQUIRED,  db_set_file, db_get_file);
+        if ( ret < 0 )
+                return ret;
         
         ret = prelude_option_add(opt, NULL, hook, 'p', PRELUDEDB_SQL_SETTING_PORT,
                                  "The port where the database server is listening (in case of client/server database)",
