@@ -67,7 +67,7 @@ static void get_failover_filename(prelude_plugin_instance_t *pi, char *buf, size
 {
         prelude_plugin_generic_t *plugin = prelude_plugin_instance_get_plugin(pi);
         
-        snprintf(buf, size, MANAGER_FIFO_DIR "/%s[%s]",
+        snprintf(buf, size, MANAGER_FAILOVER_DIR "/%s[%s]",
                  plugin->name, prelude_plugin_instance_get_name(pi));
 }
 
@@ -84,7 +84,10 @@ static int recover_from_failover(prelude_plugin_instance_t *pi, plugin_failover_
         
         do {
                 size = prelude_failover_get_saved_msg(pf->failover, &msg);
-                if ( size <= 0 )
+                if ( size < 0 )
+                        prelude_perror((prelude_error_t) size, "could not retrieve saved message from disk");
+                
+                if ( size == 0 )
                         break;
                 
                 *totsize += size;
